@@ -1,6 +1,7 @@
 import numpy as np
 import os, sys
 from tqdm import tqdm
+import paddle
 
 from ..common.paddle_utils import FeatureExtractor, extract_superpixel_features, get_pre_models
 from ..data_processor.readers import load_pickle_file
@@ -17,7 +18,6 @@ class NormLIMECVInterpreter(LIMECVInterpreter):
 
     def __init__(self,
                  paddle_model,
-                 trained_model_path,
                  model_input_shape=[3, 224, 224],
                  use_cuda=True,
                  temp_data_file='all_lime_weights.npz'):
@@ -40,8 +40,12 @@ class NormLIMECVInterpreter(LIMECVInterpreter):
         :param temp_data_file: The .npz file to save/load the dictionary where key is image path and value is another dictionary with lime weights, segmentation and input. Default: 'all_lime_weights.npz'
         :type temp_data_file: str, optional
         """
-        LIMECVInterpreter.__init__(self, paddle_model, trained_model_path,
-                                   model_input_shape, use_cuda)
+        if int(paddle.__version__[0]) > 1:
+            raise NotImplementedError(
+                "NormLIMECVInterpreter currently doesn't support paddle version 2.0 or higher"
+            )
+        LIMECVInterpreter.__init__(self, paddle_model, model_input_shape,
+                                   use_cuda)
         self.lime_interpret = super().interpret
 
         if temp_data_file.endswith('.npz'):
@@ -212,7 +216,6 @@ class NormLIMENLPInterpreter(LIMENLPInterpreter):
 
     def __init__(self,
                  paddle_model,
-                 trained_model_path,
                  use_cuda=True,
                  temp_data_file='all_lime_weights.npz'):
         """
@@ -229,8 +232,7 @@ class NormLIMENLPInterpreter(LIMENLPInterpreter):
             use_cuda (bool, optional): Whether or not to use cuda. Default: True
             temp_data_file (str, optinal): The .npz file to save/load the dictionary where key is word ids joined by '-' and value is another dictionary with lime weights. Default: 'all_lime_weights.npz'
         """
-        LIMENLPInterpreter.__init__(self, paddle_model, trained_model_path,
-                                    use_cuda)
+        LIMENLPInterpreter.__init__(self, paddle_model, use_cuda)
         self.lime_interpret = super().interpret
 
         if temp_data_file.endswith('.npz'):
